<?php
/*
* Name: UADetector 
* Description: a PHP4 class for browser & spider identification
* Version:  0.2
* Updated: 2009-04-28
* Author: Helene D. 
* Author URI: http://www.techfromhel.com/
* Usage: include_once(uadetector.class.php); 
*         $browser_info = new UADetector();
*
* Copyright (c) 2009 Helene Duncker
* Released under the GNU General Public License (GPL)
* http://www.gnu.org/licenses/gpl.txt
*/
class UADetector {
	var $name;
	var $version;
	var $os;
	var $language;
	var $device;		//PC, PDA, Phone, TV, if known
	var $model;		//Manufacturer+model, if known
	var $resolution;	//given by some embedded devices
	var $browscap;
	var $agenttype; //B=Browser, F=feed reader, H=harvester bot, 
	//M=monitor/piracy checker bot, R=archiver/search engine robot, 
	//S=Spammer, V=Validator (Link/CSS/Html)
	var $agent;

	/* constructor */
	function uaDetector($agent="") {
		$this->name = "";
		$this->version = "";
		$this->os = "";
		$this->language = "";
		$this->browscap = array();
		$this->agenttype = "";
		if (empty($agent)) { $agent = $_SERVER['HTTP_USER_AGENT']; }
		if (strlen($agent)<5) { return; }	//nothing to check
		$this->agent = $agent;

		/* for efficient browser/spider detection, test first for 
		 * the most popular user agents before less frequent 
		 * user agents. */
		$ua_info = $this->topUA($agent);
		if (empty($ua_info['name'])) { 
			$ua_info = $this->detectBrowser($agent);
		}
		if (empty($ua_info['name'])) {
			$ua_info = $this->detectSpider($agent);
		}
		if(!empty($ua_info) && is_array($ua_info)) {
			foreach($ua_info as $key => $value) {
				$this->$key = $value;
			}
		}
	} //end function uaDetector

	function topUA($agent="") {
		//Test if current or a specific user agent is a frequent
		//  (top) user agent and return a breakdown of the agent 
		//  information in an array containing Name, Version, 
		//  Language, Platform, OS, and UA agenttype (B=browser, 
		//  R=bot, F=feed/RSS reader, V=link verifier, U=utility...)

		//Top agents are based on recent log data from "WassUp", a
		//  web statistics plugin for wordpress: http://www.wpwp.org
		global $debug_mode;

		// User agent parameter or global definition is required...
		if (empty($agent)) { 
			if (!empty($this->agent)) { $agent = $this->agent; }
			else { $agent = $_SERVER['HTTP_USER_AGENT']; }
		}
		if (empty($agent)) { return null; } //nothing to check

		// Check if $agent is a top (popular) user agent. 
		$os="";
		$top_ua = array('name'=>"",'version'=>"",'os'=>"",'platform'=>"",'language'=>"",'agenttype'=>"");
		// #1 Googlebot 
		if (preg_match("#^Mozilla/\d\.\d\s\(compatible;\sGooglebot/(\d\.\d);\s\+?http\://www\.google\.com/bot\.html\)$#i",$agent,$match)>0) {
			$top_ua['name'] = "Googlebot";
			$top_ua['version']=$match[1];
			$top_ua['agenttype']= "R";
		// #2 IE 8/7/6 on Windows 2008/2007/Vista/XP/2000/2003
		} elseif (preg_match("#^Mozilla/\d\.\d\s\(compatible;\sMSIE\s(\d+)(?:\.\d+)+;\s(Windows\sNT\s\d\.\d(?:;\sW[inOW]{2}64)?)(?:;\sx64)?;?(?:\sSLCC1;?|\sSV1;?|\sGTB\d;|\sTrident/\d\.\d;|\sFunWebProducts;?|\s\.NET\sCLR\s[0-9.]+;?|\s(Media\sCenter\sPC|Tablet\sPC)\s\d\.\d;?|\sInfoPath\.\d;?)*\)$#",$agent,$match)>0) {
			$top_ua['name'] = "IE";
			$top_ua['version'] = $match[1];
			$top_ua['platform']="Windows"; 
			$os = $match[2];
			$top_ua['os'] = $this->winOSversion($os);
			$top_ua['agenttype']= "B";
			if (!empty($match[3])) {
				$top_ua['device'] = $match[3];
			}
		// #3 Firefox or other Mozilla browsers on Windows
		} elseif (preg_match("#^Mozilla/\d\.\d\s\(Windows;\sU;\s(.+);\s([a-z]{2}(?:\-[A-Za-z]{2})?);\srv\:\d(?:.\d+)+\)\sGecko/\d+\s([A-Za-z\-0-9]+)/(\d+(?:.\d+)+)(?:\s\(.*\))?$#",$agent,$match)>0) {
			$top_ua['name'] = $match[3];
			$top_ua['version'] = $match[4];
			$top_ua['language']=$match[2];
			$top_ua['platform']="Windows"; 
			$os = $match[1];
			$top_ua['os'] = $this->winOSversion($os);
			$top_ua['agenttype']= "B";
		// #4 Yahoo!Slurp
		} elseif (preg_match("#^Mozilla/\d\.\d\s\(compatible;\s(Yahoo\!\s([A-Z]{2})?\s?Slurp)/?(\d\.\d)?;\shttp\://help.yahoo.com/help/us/ysearch/slurp\)$#i",$agent,$match)>0) {
			$top_ua['name'] = $match[1];
			if (!empty($match[3])) { 
				$top_ua['version']=$match[3];
			}
			if (!empty($match[2])) { 
				$top_ua['language']=$match[2];
			}
			$top_ua['agenttype']= "R";
		// #5 MSNBot
		} elseif (preg_match("#^(?:msnbot(\-media)?)/(\d.\d)\s\(\+http\://search\.msn\.com/msnbot\.htm\)$#",$agent,$match)>0) {
			$top_ua['name'] = "MSNBot";
			if (!empty($match[1])) { 
				$top_ua['name']=$top_ua['name'].$match[1];
			}
			if (!empty($match[2])) { 
				$top_ua['version']=$match[2];
			}
			$top_ua['agenttype']= "R";
		// #6 Twiceler
		} elseif (preg_match("#^Mozilla/\d\.\d\s\(Twiceler\-(\d\.\d)\shttp://www\.cuill?\.com/twiceler/robot.html\)$#",$agent,$match)>0) {
			$top_ua['name'] = "Twiceler";
			$top_ua['version']=$match[1];
			$top_ua['agenttype']= "R";
		// #7 FeedBurner
		} elseif (preg_match("#^FeedBurner/(\d\.\d)\s\(http\://www\.FeedBurner\.com\)$#",$agent,$match)>0) {
			$top_ua['name'] = "FeedBurner";
			$top_ua['version']=$match[1];
			$top_ua['agenttype']= "F";
		// #8 Wordpress
		} elseif (preg_match("#^WordPress/(?:wordpress(\-mu)\-)?(\d\.\d+)(?:.\d+)*$#",$agent,$match)>0) {
			$top_ua['name'] = "Wordpress";
			if (!empty($match[1])) { 
				$top_ua['name']=$top_ua['name'].$match[1];
			}
			$top_ua['version']=$match[2];
			$top_ua['agenttype']= "U";
		// #9 Firefox and Gecko browsers on Mac/*nix/OS/2 etc...
		} elseif (preg_match("#^Mozilla/\d\.\d\s\((Macintosh|X11|OS/2);\sU;\s(.+);\s([a-z]{2}(?:\-[A-Za-z]{2})?)(?:-mac)?;\srv\:\d(?:.\d+)+\)\sGecko/\d+\s([A-Za-z\-0-9]+)/(\d+(?:\.[0-9a-z\-\.]+))+(?:(\s\(.*\))(?:\s([A-Za-z\-0-9]+)/(\d+(?:.\d+)+)))?$#",$agent,$match)>0) {
			$top_ua['name'] = $match[4];
			$top_ua['version'] = $match[5];
			$top_ua['language']=$match[3];
			$top_ua['platform']=$match[1];
			$os = $match[2];
			if (!empty($match[7])) { 
				$top_ua['name'] = $match[7];
				$top_ua['version'] = $match[8];
				$os=$os." ".$match[4]." ".$match[5];
			} elseif (!empty($match[6])) { 
				$os=$os.$match[6];
			}
			list($top_ua['os']) = $this->OSversion($os,$top_ua['platform'],$agent);
			$top_ua['agenttype']= "B";
		// #10 Safari and Safari-based browsers on all platforms
		} elseif (preg_match("#^Mozilla/\d\.\d\s\(([A-Za-z0-9/.]+);\sU;?\s?(.*);\s?([a-z]{2}(?:\-[A-Za-z]{2})?)?\)\sAppleWebKit/[0-9.]+\+?\s\((?:KHTML,\s)?like\sGecko\)(?:\s([a-zA-Z0-9./]+(?:\sMobile)?)/?[A-Z0-9]*)?\sSafari/([0-9.]+)$#",$agent,$match)>0) {
			$top_ua['name'] = "Safari";
			if (!empty($match[4])) { $vers = $match[4]; } 
			else { $vers = $match[5]; }
			$browser = $this->safariVersion($vers,$agent);
			if (!empty($browser) && is_array($browser)) {
				$top_ua['name'] = $browser['name'];
				$top_ua['version'] = $browser['version'];
			}
			if (empty($match[2])) {
				$os = $match[1];
			} else {
				$top_ua['platform'] = $match[1];
				$os = $match[2];
			}
			if ($top_ua['platform'] == "Windows") {
				$top_ua['os'] = $this->winOSversion($os);
			} else {
				list($top_ua['os']) = $this->OSversion($os,$top_ua['platform'],$agent);
			}
			$top_ua['language']=$match[3];
			$top_ua['agenttype']= "B";
		}
		/*
		// #11 FriendFeedBot
		} elseif (preg_match("#^Mozilla/\d\.\d\s\(compatible;\sFriendFeedBot/([0-9.]+);\s\+Http\://friendfeed\.com/about/bot\)$#",$agent,$match)>0) {
			$top_ua['name'] = "FriendFeedBot";
			$top_ua['version']=$match[1];
			$top_ua['agenttype']= "F";
		// #12 FeedFetcher Google
		} elseif (preg_match("#$#",$agent,$match)>0) {
			$top_ua['name'] = "FeedFetcher Google";
			$top_ua['version']=$match[1];
			$top_ua['agenttype']= "F";
		// #13 Baidus Spider
		} elseif (preg_match("#$#",$agent,$match)>0) {
			$top_ua['name'] = "Baidus";
			$top_ua['version']=$match[1];
			$top_ua['agenttype']= "R";
		 */
		if ($debug_mode) {
			echo "<br />\nuadetector: top_ua->name=".$top_ua['name']; //debug
		}
		return ($top_ua);
	} //end function topUA

	function detectBrowser($ua) {
		//##detect browsers
		$browser = array('name'=>"",'version'=>"",'os'=>"",'platform'=>"",'language'=>"",'agenttype'=>"B",'resolution'=>"");
		$wap = false;
		//Find MSIE browsers
		if (preg_match("/compatible(?:;|\s)+MSIE\s(\d+)(\.\d+)+(.*)/",$ua,$pcs)>0) {
			$browser['name'] = "IE";
			$browser['version'] = $pcs[1];
			$iestring = $pcs[3];

			//differentiate IE from IE-based/IE-masked browsers or spiders
			if (preg_match("/\s(AOL|America\sOnline\sBrowser)\s(\d+)(\.\d+)*/",$iestring,$pcs)>0) {
				$browser['name'] = "AOL";
				$browser['version'] = $pcs[2];

			} elseif (preg_match("#\s(Opera|Netscape)/?\s?(\d+)(\.\d+)*#",$iestring,$pcs)>0) {
				$browser['name'] = $pcs[1];
				$browser['version'] = $pcs[2];

			} elseif (preg_match("/\sAvant\sBrowser;/",$iestring,$pcs)>0) {
				$browser['name'] = "Avant";
				$browser['version'] = "";

			} elseif (preg_match("/Windows\sCE;\s?IEMobile\s(\d+)(\.\d+)*\)/i",$iestring,$pcs)>0) {
				$browser['name'] = "IEMobile";
				$browser['version'] = $pcs[1];
				$browser['os'] = "WinCE";
				$browser['platform'] = "WAP";

			//Some spiders mask as IE browser
			} elseif (preg_match("#([a-z]+(?:-|_|\s)?(?:bot|crawler|spider|reader))[^a-z](\d+(?:\.\d+)*)?#i",$ua,$pcs)>0) {
				$browser['name'] = $pcs[1];
				$browser['version'] = $pcs[2];
				$browser['agenttype']= "R";
			}

		//Find Opera browsers
		} elseif (preg_match("#Opera\sMini[/ ]([0-9\.]+)#",$ua,$pcs)>0) {
				$browser['name'] = "Opera Mini";
				$browser['version'] = $pcs[1];
		} elseif (preg_match("#Opera[/ ]([0-9\.]+)#",$ua,$pcs)>0) {
				$browser['name'] = "Opera";
				$browser['version'] = $pcs[1];

		//Find firefox-based browsers (Camino, Flock) before Firefox
		} elseif (preg_match("#[^a-z](Flock|Camino)/(\d+[\.0-9a-z]*)#",$ua,$pcs)>0) {
				$browser['name'] = $pcs[1];
				$browser['version'] = $pcs[2];
		//Find other Gecko-type browsers (incl. Firefox)
		} elseif (preg_match("#Gecko/\d+\s([a-z0-9_\- ]+)/(\d+[\.0-9a-z]*)(?:$|;?\s([a-z0-9_\- ]+)/(\d+[\.0-9a-z]*))#i",$ua,$pcs)>0) {
				$browser['name'] = $pcs[1];
				$browser['version'] = $pcs[2];
				if (!empty($pcs[3]) && stristr($pcs[3],"Firefox")!==false) {
					$browser['name'] = "Firefox";
					$browser['version'] = $pcs[4];
				} elseif (stristr($pcs[1],"Firefox")!==false) {
					$browser['name'] = "Firefox";
				}
		//Find Firefox browser
		} elseif (preg_match("#[^a-z]Firefox/(\d+[\.0-9a-z]*)#",$ua,$pcs)>0) {
				$browser['name'] = "Firefox";
				$browser['version'] = $pcs[1];
		//assume Firefox if user agent ends with Gecko only
		} elseif (preg_match("#[^a-z]Gecko/\d+$#",$ua,$pcs)>0) {
				$browser['name'] = "Firefox";

		//Find NetFront and other mobile/embedded browsers
		} elseif (preg_match("#(NetFront|NF-Browser)/([0-9\.]+)#i",$ua,$pcs)) {
				$browser['name'] = "NetFront";
				$browser['version'] = $pcs[2];
				$browser['platform'] = "WAP";	//mobile device
		} elseif (preg_match("#(Novarra-Vision)/([0-9\.]+)#i",$ua,$pcs)) {
				$browser['name'] = "Novarra-Vision";
				$browser['version'] = $pcs[2];
				$browser['platform'] = "WAP";	//mobile device
		} elseif (preg_match("#(UP.browser)/([0-9\.]+)#i",$ua,$pcs)) {
				$browser['name'] = "UP.Browser";
				$browser['version'] = $pcs[2];
				$browser['platform'] = "WAP";	//mobile device
		//Find unknown browsers that use the word "browser" in agent
		} elseif (preg_match("#([a-z0-9]+)[\- _\.]Browser[/ v\.]*([0-9\.]+)?#i",$ua,$pcs)) {
			$browser['name'] = $pcs[1];
			if (!empty($pcs[2])) {
				$browser['version'] = $pcs[2];
			} else {
				$browser['version'] = "";
			}
		}
		
		//##detect mobile/embedded devices
		//known mobile devices...
		if (preg_match('#(amoi|htc|ipaq|kindle|kwc|lge|mobilephone|motorola|nokia|PDA|Palm|Samsung|sanyo|smartphone|SonyEricsson|vodafone)[/-_ ]?([a-z]*\d+[a-z]*)*#i',$ua,$pcs)>0) {
			$browser['platform'] = "WAP";
			$browser['device'] = $pcs[1];
			if (!empty($pcs[2])) { 
				$browser['model'] = $pcs[2];
			}
			if ($pcs[1] == "KWC") { 
				$browser['device'] == "Kyocera phone";
				$browser['model'] == $pcs[0];
			}
		//check if browser header has mobile user-agent profile
		} elseif ($ua == $_SERVER['HTTP_USER_AGENT']) {
			$header_profile =array('X_WAP_PROFILE','PROFILE','13_PROFILE','56_PROFILE');
  			foreach ($header_profile AS $wap_profile) {
    				if (!empty($_ENV["HTTP_{$wap_profile}"])) {
					//has a user-agent profile header, so it's probably a mobile device
					$wap = true;
      					break 1;
				}
			}
			if ($wap && empty($browser['platform'])) { //is mobile device
				$browser['platform'] = "WAP";
			}
		} elseif (preg_match('#(J2ME/MIDP|Profile/MIDP|Danger\sHiptop)#i',$ua)>0) {
			$browser['platform'] = "WAP";
		}

		//get browser operating systems
		if (empty($browser['os']) && !empty($browser['name']) && $browser['agenttype']=="B") {
			list($browser['os'],$platform) = $this->OSversion($browser['os'],$browser['platform'],$ua);
			if (!empty($platform) && empty($browser['platform'])) {
				$browser['platform'] = $platform;
			}
		}

		//some mobile/embedded devices give screen dimensions in agent
		if (preg_match("#screen(?:res)?[ -/](\d{3,4}[x*]\d{3,4})#",$ua,$pcs)>0) {
			$browser['resolution'] = str_replace('*','x',$pcs[1]);
		} elseif (($browser['platform']=="WAP" || $browser['os']=="WAP") && preg_match("#[ ;](\d{3,4}x\d{3,4})([;)x ]|$)#",$ua,$pcs)>0) {
			$browser['resolution'] = $pcs[1];
		}

		if (!empty($browser['name']) && $browser['agenttype'] == "B" ) {
			$browser['language'] = $this->detectLanguage($ua);
		}
		return $browser;
	} //end function detectBrowser

	function detectSpider($ua) {
		$spider = array('name'=>"",'version'=>"",'os'=>"",'platform'=>"",'language'=>"",'agenttype'=>"R");
		// #FriendFeedBot
		if (preg_match("#^Mozilla/\d\.\d\s\(compatible;\sFriendFeedBot/([0-9.]+);\s\+Http\://friendfeed\.com/about/bot\)$#",$ua,$match)>0) {
			$spider['name'] = "FriendFeedBot";
			$spider['version']=$match[1];
			$spider['agenttype']= "F";
		}
		if (!empty($spider['name'])) { return $spider; }
		else { return false; }
	} //end function detectSpider

	/* determine operating system and platform from string, $os or $ua
	 * and return values for operating system and platform
	 */
	function OSversion($os,$platform="",$ua="") {
		if (empty($os)) { $os=$ua; }
		if (empty($os)) { return null; }
		$os_type = "";
		if (preg_match("/(Windows|Win|NT)[0-9; \)\/]/",$os)>0) {
			$os_type = $this->winOSversion($os);
			$platform = "Windows";
		} elseif (strpos($os,"Intel Mac OS X")!==FALSE || strpos($os,"PPC Mac OS X")!==FALSE) {
			$platform = "Macintosh";
			$os_type = "MacOSX";
		} elseif (strpos($os,"like Mac OS X")!==FALSE && !empty($platform)) {
			if (!empty($platform)) {
				$os_type = "{$platform}OSX";
			} else {
				$os_type = "MacOSX";
				$platform = "Macintosh";
			}
		} elseif (preg_match("/Android(\s[0-9.]+)?/",$os)>0) {
				$os_type = "Android";	//Google Android
				$version = $match[1];
				$platform = "WAP";
		} elseif (preg_match("/CYGWIN\s*((?:i[0-9]{3})?\s*(?:[0-9]\.[0-9]{1,2}\.[0-9]{1,2})?\s*(?:i[0-9]{3})?)?/",$os,$match)>0) {
				$os_type = "CYGWIN";
				$version = $match[1];
		} elseif (preg_match("/FreeBSD\s*((?:i[0-9]{3})?\s*(?:[0-9]\.[0-9]{1,2})?\s*(?:i[0-9]{3})?)?/",$os,$match)>0) {
				$os_type = "FreeBSD";
				$version = $match[1];
		} elseif (preg_match("/Linux\s*((?:i[0-9]{3})?\s*(?:[0-9]\.[0-9]{1,2}\.[0-9]{1,2})?\s*(?:i[0-9]{3})?)?/",$os,$match)>0) {
				$os_type = "Linux";
				$version = $match[1];
				$platform = "X11";
		} elseif (preg_match("/(PowerPC|PPC|Macintosh)/",$os)>0) {
				$os_type = "MacPPC";
				$platform = "Macintosh";
		} elseif (preg_match("/NetBSD\s*((?:i[0-9]{3})?\s*(?:[0-9]\.[0-9]{1,2}\.[0-9]{1,2})?\s*(?:i[0-9]{3})?)?/",$os,$match)>0) {
				$os_type = "NetBSD";
				$version = $match[1];
		} elseif (stristr("Nintendo Wii;",$os)!==false) {
				$os_type = "Nintendo Wii";
		} elseif (preg_match("/OpenBSD\s*([0-9\.]+)?/",$os,$match)>0) {
				$os_type = "OpenBSD";
				$version = $match[1];
		} elseif (strpos($os,"PalmOS")!==FALSE) {
				$os_type = "PalmOS";
		} elseif (preg_match("/PLAYSTATION\s(\d)/",$os,$match)>0) {
				$os_type = "Playstation";
				$version = $match[1];
		} elseif (preg_match("/IRIX\s*([0-9\.]+)?/",$os,$match)>0) {
				$os_type = "SGI IRIX";
				$version = $match[1];
		} elseif (preg_match("/SunOS\s*([0-9\.]+)?/",$os,$match)>0) {
				$os_type = "SunOS";
				$version = $match[1];
		} elseif (preg_match("/SymbianOS\/([0-9.]+)/i",$os,$match)>0) {
				$os_type = "SymbianOS";
				$version = $match[1];
		} elseif (preg_match("#Ubuntu[ /-](\d+[\.0-9a-z]*)?#i",$os)>0) {
				$os_type = "Ubuntu";	
				$platform = "X11";
		} elseif (preg_match("/\(PDA(?:.*)\)(.*)Zaurus/",$os)>0) {
				$os_type = "Zaurus";	//Sharp Zaurus
		} elseif (preg_match("#^Mozilla/\d.\d\s\((\w+);\sU;#",$os,$match)>0) {
				$os_type = $match[1];
		} elseif (!empty($platform)) {
				$os_type = $platform;
		}
		return (array($os_type,$platform));
	}  //end function OSversion

	/* determine Microsoft operating system from string, $os */
	function winOSversion($os,$platform="",$ua="") {
		if (empty($os)) { return false; }
		if (strstr($os,"Windows NT 6.2")) {
			$winos = "Win7";
		} elseif (strstr($os,"Windows NT 6.1")) {
			$winos = "Win2008";
		} elseif (strstr($os,"Windows NT 6.0")) {
			$winos = "WinVista";
		} elseif (strstr($os,"Windows NT 5.2")) {
			$winos = "Win2003";
		} elseif (strstr($os,"Windows NT 5.1")) {
			$winos = "WinXP";
		} elseif (strstr($os,"Windows NT 5.0") || strstr($os,"Windows 2000")) {
			$winos = "Win2000";
		} elseif (strstr($os,"Windows ME")) {
			$winos = "WinME";
		} elseif (preg_match("/Win(dows\s)?NT\s?([0-9\.]+)?/",$os,$match)>0) {
			$winos = "WinNT";
			if (!empty($match[2])) {
				$winos .= " ".$match[2];
			}
		} elseif (preg_match("/(?:Windows95|Windows 95|Win95|Win 95)/",$os)>0) {
			$winos = "Win95";
		} elseif (preg_match("/(?:Windows98|Windows 98|Win98|Win 98|Win 9x)/",$os)>0) {
			$winos = "Win98";
		} elseif (preg_match("/(?:WindowsCE|Windows CE|WinCE|Win CE)/",$os)>0) {
			$winos = "WinCE";
		} elseif (preg_match("/(Windows|Win)\s?3\.\d[; )\/]/",$os)>0) {
			$winos = "Win3.x";
		} elseif (preg_match("/(Windows|Win)[0-9; )\/]/",$os)>0) {
			$winos = "Windows";
		}
		if (strstr($os,"WOW64") || strstr($os,"Win64") || strstr($os,"x64")) {
			$winos = $winos." x64";
		}
		return ($winos);
	} //end function winOSversion

	/* determine Safari/Webkit-based browser and version from string, $webkit_string */
	function safariVersion($webkit="",$ua="") {
		global $debug_mode;
		$browser = "Safari";
		$vers = "";
		if ($debug_mode) {
			echo "<br />webkit=".$webkit."\n"; //debug
		}
		if (empty($webkit)) { 
			return false;
		} elseif (preg_match("#^([a-zA-Z]+)/([0-9](?:[A-Za-z.0-9]+))(\sMobile)?#",$webkit,$match)>0) {
			if ($match[1] != "Version") { //Chrome, Iron, Shiira
				$browser = $match[1];
			}
			$vers = $match[2];
			if ($vers == "0") { $vers = ""; }
			if (!empty($match[3])) { //Mobile browser
				$vers .= $match[3];
			}
		} elseif (is_numeric($webkit)) {
			$webkit_num = (int)($webkit-0.5);
			if ($debug_mode) {
				echo "<br /><!-- webkit_num = ".$webkit_num." -->\n"; //debug
			}
			if ($webkit_num > 525) { $vers = "4"; }
			elseif ($webkit_num > 419) { $vers = "3"; }
			elseif ($webkit_num > 312) { $vers = "2"; }
			elseif ($webkit_num > 99) { $vers = "1"; }
			else { $vers = ""; } //beta version, 0.x
		} //end else !empty($webkit)
		return array('name'=>$browser,'version'=>$vers);
	}  //end function safariVersion

	/* Find obvious spam bots and hackers */
	function detectSpambot($ua) {
		$crawlertype="";
		$crawler="";
		//## Find obvious script injection bots 
		if (stristr('location.href',$ua)!==FALSE) {
			$crawlertype = "spammer";
			$crawler = "Script Injection bot";
		} elseif (preg_match('/(<|&lt;|&#60;)a( |%20|&#32;|\+)href/i',$ua)>0) {
			$crawlertype = "spammer";
			$crawler = "Script Injection bot";
		} elseif (preg_match('/(<|&lt;|&#60;)script/i',$ua)>0) {
			$crawlertype = "spammer";
			$crawler = "Script Injection bot";
		} elseif (preg_match('/select*( |%20|+|%#32;)from( |%20|+|%#32;)wp_/i',$ua)>0) {
			$crawlertype = "hacker";
			$crawler = "Script Injection bot";
		}

		if (!empty($crawler)) {
			return array($crawler,$crawlertype);
		} else {
			return false;
		}
	}
	function getBrowscap($agent) {
		//check PHP browscap data for browser and platform
		if (empty($agent)) { $agent = $_SERVER['HTTP_USER_AGENT']; }
		if (empty($agent)) { return; }	//nothing to check
		$browsercap = array();
		$browser = "";
		$os = "";
		$version = "";
		if (ini_get("browscap") != "" ) {
			$browsercap = get_browser($agent,true);
			if (!empty($browsercap['platform']) && stristr($browsercap['platform'],"unknown") === false) {
				$os = $browsercap['platform'];
				if (!empty($browsercap['browser'])) {
					$browser = $browsercap['browser'];
				} else {
					$browser = $browsercap['parent'];
				}
			} else {
				$browser = $browsercap['browser'];
			}
			if (!empty($browsercap['version']) && $browscap['spider'] == FALSE) {
				$browser = trim($browser)." ".$browsercap['version'];
			}
		} else {
			return false;
		}
	} //end function getBrowscap

	function detectLanguage($agent) {
		$language="";
		if (!empty($agent)) {
		if (preg_match("/(?:\s|;|\[)(([a-z]{2})(?:\-([a-zA-Z]{2}))?)(?:;|\]|\))/",$agent,$match)>0) {
			$language = $match[1];
		}
		}
		return $language;
	}
} //end class UADetector
?>
